package com.sapuseven.untis.api.serializer

import kotlinx.serialization.KSerializer
import kotlinx.serialization.descriptors.PrimitiveKind
import kotlinx.serialization.descriptors.PrimitiveSerialDescriptor
import kotlinx.serialization.descriptors.SerialDescriptor
import kotlinx.serialization.encoding.Decoder
import kotlinx.serialization.encoding.Encoder
import java.time.Instant
import java.time.LocalDateTime
import java.time.ZoneId
import java.time.format.DateTimeFormatter
import java.time.format.DateTimeParseException

object LocalDateTimeSerializer : KSerializer<LocalDateTime> {
	private val format = DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm'Z'")

	override val descriptor: SerialDescriptor =
		PrimitiveSerialDescriptor("java.time.LocalDateTime", PrimitiveKind.STRING)

	override fun serialize(encoder: Encoder, value: LocalDateTime) {
		val string = value.format(format)
		encoder.encodeString(string)
	}

	override fun deserialize(decoder: Decoder): LocalDateTime {
		val string = decoder.decodeString()
		return try {
			LocalDateTime.parse(string, format)
		} catch (e: DateTimeParseException) {
			Instant.ofEpochMilli(0).atZone(ZoneId.systemDefault()).toLocalDateTime()
		}
	}
}
