package com.sapuseven.untis.ui.functional

import androidx.compose.runtime.Composable
import androidx.compose.runtime.DisposableEffect
import androidx.compose.runtime.getValue
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.runtime.rememberUpdatedState
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.delay
import kotlinx.coroutines.launch

@Composable
fun <T> T.useDebounce(
	delayMillis: Long = 300L,
	coroutineScope: CoroutineScope = rememberCoroutineScope(),
	onChange: (T) -> Unit
): T {
	val state by rememberUpdatedState(this)

	DisposableEffect(state) {
		val job = coroutineScope.launch {
			delay(delayMillis)
			onChange(state)
		}
		onDispose {
			job.cancel()
		}
	}
	return state
}
